#include "PulseCounter.h"

PulseCounter pulseCounter;


void PulseCounter::init(byte pin, unsigned int timeOut, byte trigger)
{
  pulseInWriteInProgress = 1;

  pulseInBit = digitalPinToBitMask(pin);
  pulseInPort = digitalPinToPort(pin);
  
  pulseInTimeOut = timeOut;
  pulseInPeriodCounter =timeOut;
  pulseInPeriod_volatile =0;
  pulseInPeriod_safe =0;
  pulseInState =0;
  pulseInTrigger = trigger;
  pulseInTotalCount_volatile = 0;
  pulseInTotalCount_safe = 0;
  
  pulseInWriteInProgress = 0;
}


//------------------------------------------------------------------------------
void PulseCounter::detectPulse()
{
  if (!pulseInWriteInProgress)
  {
    byte pulseInStateNow = (*portInputRegister(pulseInPort) & pulseInBit ? 1 : 0);
    
    // Has the state of the pulse changed?
    if ((pulseInState != pulseInStateNow) && (pulseInStateNow == pulseInTrigger))
    {
      pulseInTotalCount_volatile++;
      
      pulseInPeriod_volatile = (pulseInPeriodCounter >= pulseInTimeOut ? 0 : pulseInPeriodCounter);
      pulseInPeriodCounter = 0;
    }
    else
    {
      if (pulseInPeriodCounter < pulseInTimeOut)
      {
        pulseInPeriodCounter++;
      }
      else
      {
        pulseInPeriod_volatile = 0;
      }
    }

    if (!pulseInReadInProgress)
    {
      pulseInPeriod_safe = pulseInPeriod_volatile;
      pulseInTotalCount_safe = pulseInTotalCount_volatile;
    }
      
    if (pulseInState != pulseInStateNow)
    {
      pulseInState = pulseInStateNow;
    }
  }
}


//------------------------------------------------------------------------------
unsigned int PulseCounter::getPulseInPeriod()
{
  unsigned int period;
  
  pulseInReadInProgress = 1;
  period = pulseInPeriod_safe;
  pulseInReadInProgress =0;
  
  return period;
}


//------------------------------------------------------------------------------
unsigned long PulseCounter::getPulseInTotalCount()
{
  unsigned long count;
  
  pulseInReadInProgress = 1;
  count = pulseInTotalCount_safe;
  pulseInReadInProgress =0;
  
  return count;
}


//------------------------------------------------------------------------------
void PulseCounter::resetPulseInTotalCount()
{
  pulseInWriteInProgress = 1;
  pulseInTotalCount_volatile = 0;
  pulseInTotalCount_safe = 0;
  pulseInWriteInProgress = 0;
}
