#ifndef PULSECOUNTER_H
#define PULSECOUNTER_H

#include "Arduino.h"


class PulseCounter
{
  public:
  
    // Initializes the pulse counter. Used for counting pulses applied to an input pin. Max pulse frequency 500hz.
    void init  (byte pin = A1,                  // input pin
                unsigned int timeOut = 2500,    // the number of milliseconds to wait for a pulse, before resetting pulse in period to 0.
                byte trigger = LOW              // trigger counter on either rising or falling edge
                );
  
    // Must be called at fixed 1 ms intervals. Interrupt friendly.
    void detectPulse ();
    
    // Gets the period of the most recent pulse (in milliseconds).
    unsigned int getPulseInPeriod();
    
    // Gets the total number pulses counted.
    unsigned long getPulseInTotalCount();
  
    // Resets the pulse counter to 0.
    void resetPulseInTotalCount();
    
  private:
  
    volatile byte pulseInReadInProgress =0;
    volatile byte pulseInWriteInProgress =0;
    
    volatile unsigned int pulseInTimeOut;  // time frame for measuring pulse period.
    volatile unsigned int pulseInPeriodCounter;
    volatile byte pulseInTrigger; // trigger on LOW or HIGH
    volatile unsigned int pulseInPeriod_volatile =0;
    volatile unsigned int pulseInPeriod_safe =0;
    volatile byte pulseInState =0;
    volatile unsigned long pulseInTotalCount_volatile = 0;
    volatile unsigned long pulseInTotalCount_safe = 0;
  
    // Pulse in counter port specifics.
    uint8_t pulseInBit;
    uint8_t pulseInPort;

};


extern PulseCounter pulseCounter;

#endif

