//#include <Wire.h>
//#include <TimerOne.h>
//#include <MultiFuncShield.h>

#include "SerialCmd.h"
#include "Config.h"
#include "PulseCounter.h"

float currentSpeedKmh = 0;
unsigned int currentRevsPerMin = 0;
float currentAccelMss = 0; // m/s/s
unsigned int wheelCircumferenceCm = 0;

unsigned int periodMs = 0;
unsigned long lastCalcTime100 = 0;
unsigned long lastCalcTime1000 = 0;
unsigned long lastDetectTime = 0;

char timedOut = 0;

#define PulseInPin  5
#define AlarmPin    3
#define speedKmh(periodMs)    ((float)(wheelCircumferenceCm * 36) / (unsigned int) periodMs)
#define revsPerMin(periodMs)  ((unsigned int)60000 / (unsigned int) periodMs)


void setup() {
  pinMode(0, INPUT_PULLUP); // Initialize serial RX pin.
  pinMode(PulseInPin, INPUT_PULLUP);
  pinMode(AlarmPin, OUTPUT);
  digitalWrite(AlarmPin, HIGH);
  
  MySerial.begin(9600);   // MySerial is a macro that resolves to either Serial or Serial1.
  
  currentConfig.load();
  wheelCircumferenceCm = ((PI * currentConfig.wheelDiameterMm) + 5) / 10;   // Calculate to nearest cm.

  pulseCounter.init(PulseInPin);
  
  //Timer1.initialize();
  //MFS.initialize(&Timer1);
  //MFS.setDisplayBrightness(3);
}


void loop() {
  serialCmd.serialEvent();
  
  const unsigned long microsLocal = micros();
 
  if (microsLocal - lastDetectTime >= 1000)
  {
    lastDetectTime = microsLocal;
    pulseCounter.detectPulse();
  }
  
  if (microsLocal - lastCalcTime100 >= 100000) // calculate speed every 100 milliseconds
  {
    lastCalcTime100 = microsLocal;

    periodMs = pulseCounter.getPulseInPeriod();
    
    if (periodMs > 0)
    {
      timedOut = 0;
      float lastSpeedKmh = currentSpeedKmh;
      
      currentSpeedKmh = speedKmh(periodMs);
      currentRevsPerMin = revsPerMin(periodMs);
      currentAccelMss = ((currentSpeedKmh - lastSpeedKmh) * 100) / 36;
  
      if (currentSpeedKmh > currentConfig.tripMaxSpeedKmh)
      {
        currentConfig.tripMaxSpeedKmh = currentSpeedKmh;
      }
    }
    else
    {
      periodMs = 0;
      currentSpeedKmh = 0;
      currentRevsPerMin = 0;
      currentAccelMss = 0;
      
      if (!timedOut)
      {
        currentConfig.save();
      }
      timedOut = 1;
    }
    //MFS.write(currentSpeedKmh);
  }
  
  if (microsLocal - lastCalcTime1000 >= 1000000) // calculate distance every 1000 milliseconds
  {
    lastCalcTime1000 = microsLocal;

    if (periodMs > 0)
    {
      unsigned int distanceTravelledMetres = ((((unsigned long)wheelCircumferenceCm * 1000) / periodMs) + 50) / 100;  // distance (rounded to nearest metre)

      currentConfig.totalDistanceMetres += distanceTravelledMetres;
      currentConfig.tripDistanceMetres += distanceTravelledMetres;

      currentConfig.totalTimeSeconds++;
      currentConfig.tripTimeSeconds++;
    }

    char alarmTrigger = (currentSpeedKmh > currentConfig.speedLimitKmh) ? 1 : 0;
    
    digitalWrite(AlarmPin, !alarmTrigger);
  }


  // Process serial commands.
  if (serialCmd.complete)
  {
    switch (serialCmd.cmd)
    {
      case CMD_READ_DATA:
        MySerial.print(currentSpeedKmh);
        MySerial.print(",");
        MySerial.print(currentAccelMss);
        MySerial.print(",");
        MySerial.print(currentRevsPerMin);
        MySerial.print(",");
        MySerial.print(currentConfig.totalDistanceMetres);
        MySerial.print(",");
        MySerial.print(currentConfig.totalTimeSeconds);
        MySerial.print(",");
        MySerial.print(currentConfig.tripDistanceMetres);
        MySerial.print(",");
        MySerial.print(currentConfig.tripTimeSeconds);
        MySerial.print(",");
        MySerial.println(currentConfig.tripMaxSpeedKmh);
        break;
      case CMD_RESET_TRIP:
        currentConfig.tripDistanceMetres = 0;
        currentConfig.tripTimeSeconds = 0;
        currentConfig.tripMaxSpeedKmh = 0;
        currentConfig.save();
        MySerial.println("ok");
        break;
      case CMD_READ_CONFIG:
        MySerial.print((float)currentConfig.wheelDiameterMm/10);
        MySerial.print(",");
        MySerial.println(currentConfig.speedLimitKmh);
        break;
      case CMD_WRITE_CONFIG:
        currentConfig.wheelDiameterMm = constrain(serialCmd.args.configArgs.wheelDiameterMm, 30, 1000);
        currentConfig.speedLimitKmh = constrain(serialCmd.args.configArgs.speedLimitKmh, 5, 240);
        currentConfig.save();
        MySerial.println("ok");
        break;
      case CMD_ERR:
        MySerial.println("err");
        break;
    }
    serialCmd.reset();
  }
}


// -------------------------------------------------------------

